import time
import unittest

from mandarina.benchmark import Benchmark


class TestBenchmark(unittest.TestCase):
    def test(self):
        pass

    def test_benchmark(self):
        execution_time = 0.1
        result = Benchmark.run(lambda: time.sleep(execution_time), 10, print_output=False)
        self.assertAlmostEqual(result[0], execution_time, 2)
        self.assertAlmostEqual(result[1], execution_time, 2)



"""
This module contains benchmarking tools for functions.
"""

import random
import time
import sys
import statistics


class Benchmark:
    """
    This class runs a benchmark on a function passed to the
    run method. It calls the function a specified number of times
    and calculates the mean and standard deviation across all runs.
    """

    @staticmethod
    def run(function, runs, print_output=True):
        """
        The run method prints the benchmark statistics when called
        with a function as parameter. If you set print_output to False,
        the function will only return the final value without printing
        anything to the console.

        :param function: The function to benchmark
        :return: The mean running time in seconds and its standard
                 deviation.

        Example
        >>> Benchmark.run(lambda: time.sleep(1), 100)

        """
        timings = []
        if print_output:
            print("Runs Median Mean Stddev")
        for i in range(runs):
            startTime = time.time()
            function()
            seconds = time.time() - startTime
            timings.append(seconds)
            median = statistics.median(timings)
            mean = statistics.mean(timings)
            if print_output:
                if i < 10 or i % 10 == 9:
                    print(
                        "{0}\t{1:3.2f}\t{2:3.2f}\t{3:3.2f}".format(
                            1 + i,
                            median,
                            mean,
                            statistics.stdev(timings, mean) if i > 1 else 0,
                        )
                    )
        return (median, mean, statistics.stdev(timings, mean))



"""
This module provides functionality to work with files and folders.
"""

import os

def create_dir_if_doesnt_exist(folderpath):
    """
    Creates a folder if it doesn't exist.

    :param folderpath:
    :return: True if folder was created, else False

    """
    if not os.path.exists(folderpath):
        os.makedirs(folderpath)
        return True
    return False

def delete_file(filepath):
    """
    Deletes the specified file if the file exists.

    :param filepath: Path of the file to be deleted
    :return: True if the file was deleted, else False

    """
    if os.path.isfile(filepath):
        os.remove(filepath)
        return True
    else:
        print("Error: %s file not found" % filepath)
        return False

def delete_dir(dirpath):
    """
    Deletes the specified directory if it exists

    :param dirpath: Path to the directory
    :return: True if the directory was deleted, else False

    """
    if os.path.isdir(dirpath):
        os.rmdir(dirpath)
        return True
    else:
        print("Error: %s directory not found" % dirpath)
        return False

def count_files_in_dir(dirpath):
    """
    Counts the number of files contained in the specified
    directory.

    :param dirpath: Path to the directory
    :return: Number of files

    """
    return len([name for name in os.listdir(dirpath)
                       if os.path.isfile(os.path.join(dirpath, name))])
